/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: base64.c (for base64 encoding/decoding)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"

#define MAX_ENCODING_BUFFER_SIZE    20000
char base64_table[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

int Base64_Decode_byte(int b64_byte)
{
    if (b64_byte >= 'A' && b64_byte <= 'Z')
        return b64_byte - 'A';
    
    if (b64_byte >= 'a' && b64_byte <= 'z')
        return b64_byte - 'a' + 26;
    
    if (b64_byte >= '0' && b64_byte <= '9')
        return b64_byte - '0' + 52;
    
    if (b64_byte == '+')
        return 62;
    
    return 63;
}

char *Base64_Encode(const char *buffer)
{
    char *encoded_buffer, *part, *old_part;
    int len = strlen(buffer), part_location = 0;
    int a = 0, b = 0, c = 0, d = 0, char_count = 0, part_size = 0;
    
    encoded_buffer = (char *) malloc((strlen(buffer)/4)*6);
    part =     (char *) malloc(4);
    old_part = (char *) malloc(3);

    memset(encoded_buffer, 0, (strlen(buffer)/4)*6);
    memset(part, 0, 4);
    memset(old_part, 0, 3);

    while (buffer[part_location])
    {
        part_size = 4;
        old_part[0] = buffer[part_location];
        
        /* check if PART size exceeds remainder of
           bits to check */
        if (part_size > (len - part_location))
            part_size = len - part_location;          
        
        /* Make sure the new bytes are existant
           before grabbing them */
        if (part_size > 1)
            old_part[1] = buffer[part_location + 1];
        if (part_size > 2)
            old_part[2] = buffer[part_location + 2];
        
        /* Transform bits */
        a = old_part[0] >> 2;
        b = ((old_part[0] & 0x3) << 4) | (old_part[1] >> 4);
        
        if (part_size > 1)
            c = ((old_part[1] & 0xf) << 2) | (old_part[2] >> 6);
        
        if (part_size > 2)
            d = (old_part[2] & 0x3f);
        
        
        /* Translate and append to encoded_buffer */
        part[0] = base64_table[a];
        part[1] = base64_table[b];
        if (part_size > 1)            
            part[2] = base64_table[c];
        else 
            part[2] = '=';
        if (part_size > 2)
            part[3] = base64_table[d];
        else
            part[3] = '=';

        strcat(encoded_buffer, part);

        /* Max. line width is 74 bytes, start new line after that */        
        char_count++;
        if (char_count == 18)
        {
            encoded_buffer[strlen(encoded_buffer)] = '\n';
            char_count = 0;
        }
        
        if (part_size < 4)
            break;

        memset(part, 0, 4);
        part_location += 3;
    }
    
    return encoded_buffer;
}

char *Base64_Decode(const char *buffer)
{
    char *decoded_buffer = (char *)malloc(strlen(buffer));
    char *part = (char *)malloc(4);
    int a = 0, b = 0, c = 0, d = 0, c2 = 0, d2 = 0;
    int part_location = 0;
    
    memset(decoded_buffer, 0, strlen(buffer));
    
    while (buffer[part_location])
    {
           if (buffer[part_location] == '\n')
               part_location++;
           a = buffer[part_location];
           b = buffer[part_location + 1];
           c = buffer[part_location + 2];
           d = buffer[part_location + 3];

           /* Decode each byte according to RFC */
           a = Base64_Decode_byte(a);
           b = Base64_Decode_byte(b);
           c2 = c; /* Make copies of bytes 3 and 4 of each part */
           d2 = d;
           if (c != '=')
               c = Base64_Decode_byte(c);
           if (d != '=')
               d = Base64_Decode_byte(d);

           /* Transform bits back to normal */
           part[0] = (a << 2) | (b >> 4);
           if (c2 != '=')
           part[1] = ((b & 0xf) << 4) | (c >> 2);
           if (d2 != '=')
           part[2] = ((c & 0x3) << 6) | d;
           
           /* Look back and check if the copies of 3 and 4 are
              set as '=' if so, NULL them out */
           if (c2 == '=') 
               part[1] = '\0';
           if (d2 == '=')
               part[2] = '\0';
           
           
           strcat(decoded_buffer, part);
           part_location += 4;     
    }
    
    return decoded_buffer;
}

/* EOF */

