/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: conf.c (configuration file parser)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"

#define MAX_LINE_LENGTH      255
#define MAX_VARIABLE_LENGTH  48
#define MAX_VALUE_LENGTH     255

extern struct config_options config_opt[];

int Load_Configuration(char *path)
{
    FILE *cfg_fd;
    char *line, *p, *q, *variable, *value, *garbage;
    int line_num, i, x;
    
    /* Open configuration file */
    if ((cfg_fd = fopen(path, "r")) == NULL)
    {
        perror("fopen()");
        return -1;
    }
    
    /* Allocate memory for the line buffer */
    line = (char *)malloc(MAX_LINE_LENGTH);
    memset(line, 0, MAX_LINE_LENGTH);
    
    /* Allocate memory for the other variables */
    variable = (char *)malloc(MAX_VARIABLE_LENGTH);
    value    = (char *)malloc(MAX_VALUE_LENGTH);
    garbage  = (char *)malloc(MAX_LINE_LENGTH);
    
    line_num = 0;
    while (fgets(line, MAX_LINE_LENGTH, cfg_fd) != 0)
    {
        line_num++;
        
        memset(variable, 0, strlen(variable));
        memset(value, 0, strlen(value));
        
        /* Ignore comments and empty lines */
        if (*line == '#' || !*line)
            continue;
        
        /* Filter out pounds and newlines */
        if ((p = strchr(line, '#')))
                *p = 0;
        if ((p = strchr(line, '\n')))
                *p = 0;
        
        q = line;
        if (line[0] == '\0' || *q == '\0')
            continue;
        
        if ((p = strchr(line, '=')))
        {
            *p++;
            strcpy(value, p);
            
            /* Remove unwanted spaces */
            while (value[0] == ' ')
                value++;
            
            if (value[0] == '\"')
            {
                value++; /* Get out of that left quote */
                
                strcpy(garbage, value);
                if (strtok(garbage, "\"") == NULL)
                {
                    printf("Config file error.. "
                            "Missing right quotation mark line %d\n",
                            line_num);
                    continue;
                }
                
              
                strcpy(value, garbage);
            }
            else
            {    /* My configs always require quoted
                    values, it's out of habit */
                 printf("Config file syntax error line %d\n",
                         line_num);
                 continue;   
            }
        }
        
        /* Time to extract the variable */
        strcpy(garbage, line);
        strtok(garbage, "=");
        
        /* Ignore all spaces, and convert letters
           to Uppercase */
        for (i = 0, x = 0; i < strlen(garbage); i++)
        {
            if (garbage[i] != ' ')
            {
                variable[x] = (char) toupper(garbage[i]);
                x++;   
            }   
        }
        
        if (variable && value)
        {
            if (!strcmp(variable, "SUSPECT_BUFFER_SIZE"))
            {
                config_opt[C_OPT_SUSPECT_BUFFER].value = atoi(value);
                continue;
            }
            
            if (!strcmp(variable, "ENABLE_LOGGING"))
            {
                config_opt[C_OPT_ENABLE_LOGGING].value = atoi(value);
                continue;
            }   
            
            if (!strcmp(variable, "LOGFILE"))
            {
                strncpy(config_opt[C_OPT_LOGFILE].option,
                         value, strlen(value));
                continue;   
            }
            
            if (!strcmp(variable, "OUR_BUFFER_SIZE"))
            {
                config_opt[C_OPT_OUR_BUFFER_SIZE].value = atoi(value);
                continue;
            }
            
            if (!strcmp(variable, "CAPTURE_DIRECTORY"))
            {
                strncpy(config_opt[C_OPT_CAPTURE_DIRECTORY].option,
                        value, strlen(value));
                continue;
            }

            if (!strcmp(variable, "SHELLCODE_DEFINITIONS"))
            {
                strncpy(config_opt[C_OPT_SHELLCODE_DEFS].option,
                        value, strlen(value));
                continue;
            }
            
            printf("Config Warning: Unknown variable '%s', "
                    "ignoring\n", variable);
        }
    }
    
    free(line);
    fclose(cfg_fd);
    
    return 0;
}

/* EOF */
