/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: main.c (init)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"
#include "messages.h"
#include "version.h"
#include "getopt.h"

int Server_port, Use_SHCD = 0, sniffer = 0;
int Disable_hexdump = 0;
char *Script_File;
/*
** Display Usage details / Help context
*/
int Main_Usage(char *appname)
{
   extern char *Device;
   printf("Usage: %s [options ....]\n", appname);

   printf("\nOptions\n"
	  "  --config=PATH             Configuration file location (%s)\n"
	  "  -s, --script=PATH         Perl script for server behaviour\n"
	  "  -p, --port=PORT           Specify port to use (required)\n"
	  "  -u, --unusual=LENGTH      Specify a buffer size you think is suspect\n"
	  "  -d, --display             Display configuration while loading\n"
	  "  -v, --version             Display version information and exit\n",
	  DEFAULT_CONFIG_FILE);

#ifdef HAVE_LIBPCAP
   Find_Network_Device();
   printf("\nPacket sniffing\n"
	  "  --sniff                   Enable packet sniffing mode\n"
	  "  --device=<device>         Device to sniff (Default: %s)\n", Device);
#endif

   printf("\nAdditional options\n"
	  "  --raw-capture             Write raw data to capture file\n"
	  "  --disable-hex-dump        Buffers containing unprintable characters\n"
	  "                            will not be written to stdout\n"
   "  --display-opcodes         Display matching operation codes in suspicious buffers\n");

printf("\nHelp options\n"
       "  -h, --help                Show this message\n\n");

return 0;
}

/*
** Incase filename's contain '~' instead of the full path to a user's home directory,
** do some modification.
*/
char *Main_Expand_path(char *filename)
{
   char *new, *tmp;
   struct passwd *usr_info;

    /* Make sure we need to expand this */
   if (filename[0] != '~')
     return filename;
   *filename++;

   if (filename[0] != '/')
     {
        /* Someone else's home dir was specified,
           syntax is ~user/path */
        if (!strchr(filename, '/'))
	  return NULL;

        tmp = (char *) malloc(strlen(filename));
        strcpy(tmp, filename);
        strtok(tmp, "/");

        usr_info = getpwnam(tmp);

        if (!(filename = strchr(filename, '/')))
	  return NULL;
     }
   else
     usr_info = getpwuid(getuid());

    /* Add the expanded path to the original */
   new = (char *) malloc(FILENAME_MAX);
   sprintf(new, "%s%s", usr_info->pw_dir, filename);

   return new;
}

/*
** Initialize application
*/
int main(int argc, char *argv[])
{
   int alt_conf = 0;
   char c;
   char *Config_File;

#ifdef HAVE_LIBPCAP
   extern char *Device;
   extern int Selected_Device;
#endif

   struct stat script_stat[1];
   struct stat conf_stat[1];

   extern char *optarg;
   extern int Suspect_Size;
   extern int Display_conf_setup;
   extern int Raw_mode;
   extern int Use_Script;
   extern int show_matching_opcodes;

   extern struct config_options config_opt[];

    /* Allocate PATHs for Configuration and Script */
   if (!(Config_File = (char *)malloc(FILENAME_MAX)) ||
       !(Script_File = (char *)malloc(FILENAME_MAX)))
     {
        fprintf(stderr, ERROR_MALLOC);

        return 1;
     }

    /* Set Config_File before get_opt**() is called so
       it doesn't get replaced by accident */
   Config_File = Main_Expand_path(DEFAULT_CONFIG_FILE);

    /* Check options */
   while (1)
     {
        static struct option long_options[] =
	  {
	       { "config",           required_argument, 0,  0 },
	       { "port",             required_argument, 0, 'p' },
	       { "script",           required_argument, 0, 's' },
	       { "unusual",          required_argument, 0, 'u' },
	       { "raw-capture",      no_argument,       0,  3  },
	       { "display",          no_argument,       0, 'd' },
	       { "version",          no_argument,       0, 'v' },
	       { "help",             no_argument,       0, 'h' },
	       { "disable-hex-dump", no_argument,       0,  4  },
	       { "display-opcodes",  no_argument,       0,  5  },
#ifdef HAVE_LIBPCAP
	       { "sniff",     no_argument,       0,  1  },
	       { "device",    required_argument, 0,  2  },
#endif
	       { 0, 0, 0, 0 }
	  };

        int  option_index = 0;

        if ((c = getopt_long(argc, argv, "rvd:p:s:u:h?", long_options,
			     &option_index)) == -1)
	  break; /* End of options */

        switch (c)
	  {
	   case 0: /* Config file */
	     alt_conf = 1;
	     Config_File = Main_Expand_path(optarg);
	     break;
#ifdef HAVE_LIBPCAP
            /* Enable flag for packet sniffing */
	   case 1:
	     sniffer = 1;
	     break;
            /* Manually set device to sniff */
	   case 2:
	     Selected_Device = 1;
	     if (optarg)
	       Device = optarg;
	     break;
#endif

            /* Server port */
	   case 'p':
	     Server_port = atoi(optarg);
	     break;
                       /* Perl script */
	   case 's':
	     Script_File = Main_Expand_path(optarg);
	     Use_Script = 1;
	     break;
                       /* Suspicious buffer size (start.) */
	   case 'u':
	     Suspect_Size = atoi(optarg);
	     break;
                       /* Raw buffer saving */
	   case 3:
	     printf(OPTION_RAW);
	     Raw_mode = 1;
	     break;
                       /* Disable hexdumps */
	   case 4:
	     Disable_hexdump = 1;
	     break;
                       /* Display setup */
	   case 'd':
	     Display_conf_setup = 1;
	     break;
                       /* Display version */
	   case 'v':
	     printf("%s\n", VERSION);
	     return 0;
	   case 5:
	     show_matching_opcodes = 1;
	     break;

	   default: /* Usage / Help context */
	     Main_Usage(argv[0]);
	     return 1;
	  }

     }

   printf(IMPOST_BANNER, VERSION);

    /* We need a port */
   if (!Server_port)
     {
        Main_Usage(argv[0]);

        return 1;
     }

    /* Load shellcode definitions */
   if (SHC_Parse_Defs_File() == -1)
     return 1;

    /* Check to see if the config file exists, if not
       ignore it and use default settings */
   if (lstat(Config_File, conf_stat) == -1)
     printf(ERROR_CONFIG_NOTFOUND, Config_File);
   else
     {
        if (alt_conf == 1)
	  printf(OPTION_CONFIG, Config_File);

        Load_Configuration(Config_File);
     }

   if (Suspect_Size)
     config_opt[C_OPT_SUSPECT_BUFFER].value = Suspect_Size;

#ifdef HAVE_LIBPCAP
    /* Enable packet sniffing instead of the server */
   if (sniffer)
     {
	Impost_Sniff_Loop(Server_port);

	return 0;
     }
#endif

   if (strlen(Script_File) > 0)
     {
        /* Make sure the script exists before
           executing it */
        if (lstat(Script_File, script_stat) == -1)
	  printf(ERROR_SCRIPT_NOTFOUND, Script_File);

        else
	  {
	     printf(IMPOST_LOAD_SCRIPT, Script_File);

	     if (Perl_Load_Script(Script_File))
	       {
		  fprintf(stderr, ERROR_SCRIPT_FAIL);

		  return 1;
	       }
	  }
     }

   Impost_Create_Server();
   Impost_Server_Loop();

   return 0;
}

/* EOF */
