/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: packets.c (packet sniffing routines)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"
#include "messages.h"

extern int Detected_NOP_chr;
extern int Has_Opcode_Instructions;
extern struct config_options config_opt[];

/*
** Find network device
*/
int Find_Network_Device()
{
    /* let libpcap do all the work */
   Device = pcap_lookupdev(errbuff);
   pcap_lookupnet(Device, &Ipaddr, &Netmask, errbuff); /* get the ip and netmask */

   if (Device == NULL)
     {
        fprintf(stderr, "pcap_lookupnet(): %s\n", errbuff);

        exit(1); /* oops */
     }

   return 0;
}

/*
** Find the datalink type and set the offset for the device we're using.
*/
int Set_Datalink_Offset(pcap_t *PC_S)
{
    /* Decide datalink offset based on pcap_datalink()'s return value */
   switch(pcap_datalink(PC_S))
     {
      case DLT_NULL:
	dl_offset = 4;
	break;
      case DLT_EN10MB:
      case DLT_EN3MB:
	dl_offset = 14;
	break;
      case DLT_PPP:
	dl_offset = 4;
	break;
      case DLT_SLIP:
	dl_offset = 16;
	break;
      case DLT_FDDI:
	dl_offset = 21;
	break;
      case DLT_RAW:
	dl_offset = 0;
	break;
      default: /* Very unlikely, let's hope this isn't the case */
	fprintf(stderr, ".x unknown data link -- %d!\n",
                pcap_datalink(PC_S));
	exit(1);
     }

   return 0;
}

/*
** Extract headers and payload from a packet
*/
int Decode_Packet()
{
   u_short ip_len, tcp_len;

    /* Extract IP header */
   ip_hdr = (struct siphdr *)(packet + dl_offset);
   ip_len = (ip_hdr->ihl << 2);
   if (ip_hdr->protocol != 6) /* We're only going to fuck with TCP */
     return -1;

    /* Extract TCP header */
   tcphdr = (struct stcphdr *)(packet + dl_offset + ip_len);
   tcp_len = (tcphdr->th_off << 2);

    /* Extract payload */
   payload = (char *)(packet + dl_offset + ip_len + tcp_len);

   return 0;
}

/*
** Main program loop if Impost was told to sniff packets
** instead of acting like a server.
*/
int Impost_Sniff_Loop(int dest_port)
{
   int NOPcount = 0, nop_start = 0, f_Instruct = 0;
   int data_seg_len, noprint = 0, i, hex;

   if (Selected_Device != 1)
     Find_Network_Device();

   printf("%s INITIALIZING PACKET SNIFFER\n",
	  Impost_Get_TimeStamp());
   Session = pcap_open_live(Device, 9999, 1, 1000, errbuff);
   if (Session == NULL)
     {
        fprintf(stderr, "\n.x pcap_open_live() -- %s\n", errbuff);
        exit(1);
     }

   Set_Datalink_Offset(Session);
   printf(".> Device: %s\n"
	  ".> Destination Port: %d\n", Device, dest_port);

   if (!(buffer = (char *)malloc(9999)))
     {
        fprintf(stderr, ERROR_MALLOC);

        exit(1);
     }

   while (1)
     {
        /* Read next packet */
        packet = pcap_next(Session, &header);
        if (packet != NULL)
	  {
            /* Wrong protocol */
	     if (Decode_Packet() == -1)
	       continue;

            /* If the destination port is the same as the one specified
               on startup, let's look at the packet */
	     if (ntohs(tcphdr->th_dport) == dest_port)
	       {
		  saddr.s_addr = ip_hdr->saddr;
		  switch(tcphdr->th_flags)
		    {
		     case TH_FIN|TH_ACK:
		       printf(".. Packet with flags TH_FIN|TH_ACK from %s (Bye)\n",
			      inet_ntoa(saddr));
		       break;
		     case TH_ACK:
		       printf(".. Received ACK: %d from %s\n",
			      ntohs(tcphdr->th_ack), inet_ntoa(saddr));
		       break;
		    }

                /* Let's keep things simple, if TH_PUSH isn't set let's move
                   onto the next packet */
		  if (!(tcphdr->th_flags&TH_PUSH))
                    continue;

		  printf(".. Packet, with TH_PUSH flag, from %s\n",
			 inet_ntoa(saddr));
#ifdef DEBUG
                /* Dump more information on debug mode */
		  printf(".d ip_hdr->protocol..: %d\n", ip_hdr->protocol);
		  printf(".d ip_hdr->tot_len...: %d\n", ntohs(ip_hdr->tot_len));
		  printf(".d tcphdr->th_flags..: 0x%x\n", tcphdr->th_flags);
		  printf(".d  -TH_FIN.........: %s\n"
			 ".d  -TH_SYN.........: %s\n"
			 ".d  -TH_PUSH........: %s\n"
			 ".d  -TH_ACK.........: %s\n",

			 tcphdr->th_flags&TH_FIN  ? "YES":"NO",
			 tcphdr->th_flags&TH_SYN  ? "YES":"NO",
			 tcphdr->th_flags&TH_PUSH ? "YES":"NO",
			 tcphdr->th_flags&TH_ACK  ? "YES":"NO"
			 );
#endif
		  data_seg_len = (ntohs(ip_hdr->tot_len)-(ip_hdr->ihl<<2)-(tcphdr->th_off<<2));
		  printf(".i SEQ = %d ACK = %d\n",
			 ntohs(tcphdr->th_seq), ntohs(tcphdr->th_ack));
		  printf(".i Payload -- %d bytes\n", strlen(payload));
		  printf(".i Payload's data segment is %d bytes\n",
			 data_seg_len);

		  if ((tcphdr->th_flags&TH_PUSH) && (tcphdr->th_flags&TH_ACK))
		    {
		       memset(buffer, 0, 9999);
		       strncpy(buffer, payload, data_seg_len);

		       if (buffer[data_seg_len-1] == '\n' || buffer[data_seg_len-1] == 'r')
			 buffer[data_seg_len-1] = '\0';

		       /* Check buffer for non-printable characters, do we need
                       to run a hexdump instead of printing directly to STDOUT? */
		       for (i = 0; i < strlen(buffer)-1; i++)
			 {
			    if (!isprint(buffer[i]) && buffer[i] != '\n' && buffer[i] != '\r')
			      {
				 noprint = 1;
				 break;
			      }
			    noprint = 0;
			 }

		       /* Look for opcode instructions */
		       Has_Opcode_Instructions = SHC_Has_X86_Opcode(buffer);
		       if (Has_Opcode_Instructions)
			 {
			    /* Check for x86 NOP instructions */
			    NOPcount = Check_x86_NOPs(buffer);
			    if (NOPcount > 48)
			      {
				 Detected_NOP_chr = 0x90;
				 nop_start = Find_1byte_NOP_Start(buffer, Detected_NOP_chr);
			      }

			    /* Find ending of padding, usually the first instruction */
			    if (NOPcount > 48)
			      {
				 i = nop_start;
				 while (buffer[i])
				   {
				      hex = (int)(buffer[i] & 0xff);
				      if (hex != Detected_NOP_chr)
					break;

				      f_Instruct++;
				      i++;
				   }
			      }
			 }

		       /* Is the buffer suspicious? */
		       if ((data_seg_len>config_opt[0].value) || (Has_Opcode_Instructions))
			 {
			    Detected_Buffer(buffer, data_seg_len, NOPcount,
					    nop_start, 0, f_Instruct);
			 }

		       if (!noprint)
			 printf(".i DATA SEGMENT: %s\n", buffer);

		       else
			 Impost_Hexdump(buffer, 0);
		    }
	       }
	  }
     }

   return 0;
}

/* EOF */
