/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: script.c (handles perl embedding)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"
#include "messages.h"

#include <EXTERN.h>
#include <perl.h>

static PerlInterpreter *my_perl;

/*
** Load Perl script, should work smoothly unless you suck at Perl
*/
int Perl_Load_Script(char *Script_Path)
{
    char *Script_Args[] = { "", Script_Path };
    
    my_perl = perl_alloc();
    perl_construct(my_perl);

    perl_parse(my_perl, NULL, 2,
        Script_Args, NULL);
    
    perl_run(my_perl);
    
    /* If script has problems, return -1 */
    if (SvTRUE(ERRSV))
        return -1;

    return 0;
}

/*
** Unload Perl script and free PerlInterpreter
*/
int Perl_Unload_Script()
{
    perl_destruct(my_perl);
    perl_free(my_perl);

    return 0;
}

/*
** Call 'on_server_accept' sub routine in perl script
** to see how to properly welcome new client
*/
int Perl_On_Server_Accept(char *IP_Address, int fd)
{
    char   *Buffer;
    
    STRLEN  n_a;
    I32     Welcome_Line_count, i;
    AV     *Welcome_message;

    if (!(Buffer = (char *)malloc(256)))
    {
        printf(ERROR_MALLOC);
        
        exit(1);   
    }
    
    dSP;
    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
    
    /* $_[0] for sub on_server_accept */
    XPUSHs(sv_2mortal(newSVpv(IP_Address, 0))); 
    XPUSHs(sv_2mortal(newSViv(fd))); /* $_[1] */
    PUTBACK;
    
    /* Call on_server_accept sub routine */
    call_pv("on_server_accept", G_DISCARD);

    FREETMPS;
    LEAVE;

    Welcome_message = get_av("welcome_message", TRUE);
    Welcome_Line_count = av_len(Welcome_message) + 1;
    for (i = 0; i < Welcome_Line_count; i++)
    {
#ifdef DEBUG
        printf("$welcome_message[%d] = '%s'\n",
            i, SvPV(*av_fetch(Welcome_message,
            i, FALSE), n_a));
#endif
        strncpy(Buffer, SvPV(*av_fetch(Welcome_message,
            i, FALSE), n_a), 256);

        Buffer[strlen(Buffer)+1] = '\n';
        printf("io send(sock=%d,size=%d): %s", fd, strlen(Buffer), Buffer);
        send(fd, Buffer, strlen(Buffer), 0);
    }
    

    av_clear(Welcome_message);
    
    if (Buffer != NULL)
        free(Buffer);
    Buffer = NULL;

    return 0;
}

/*
** Call 'on_server_read' sub routine in perl script
** to decide how to respond to client's input
*/
int Perl_On_Server_Read(char *Buffer, int fd)
{
    char    *Buf;
    AV      *Reply_data;
    SV      *sv_data;   
    STRLEN   n_a;
    I32      Reply_Line_count;
    I32      D_Notice;    
    I32      i;
    
    if (!(Buf = (char *)malloc(256)))
    {
        printf(ERROR_MALLOC);
        
        exit(1);   
    }
    
    dSP;
    ENTER;
    SAVETMPS;

    PUSHMARK(SP); /* Send Read() buffer to 'on_server_read', it will be $_[0] */
    XPUSHs(sv_2mortal(newSVpv(Buffer, 0)));
    XPUSHs(sv_2mortal(newSViv(fd)));
    PUTBACK;
    call_pv("on_server_read", G_DISCARD);
    FREETMPS;
    LEAVE;

#ifdef DEBUG
    printf(".d Called 'on_server_read' perl routine, grabbing @reply_data\n");
#endif    
        
    /* Get reply array */
    Reply_data = get_av("reply_data", TRUE);
    Reply_Line_count = av_len(Reply_data) + 1;
#ifdef DEBUG
    printf(".d Got @reply_data, time to chop it up and send it out\n");
#endif
    for (i = 0; i < Reply_Line_count; i++)
    {
#ifdef DEBUG
    printf(".d Attempting to send $reply_data[%d]...\n", i);
#endif
//        if (strlen(Buf) > 0)
        memset(Buf, 0, 256);
        
        strncpy(Buf, SvPV(*av_fetch(Reply_data,
            i, FALSE), n_a), 256);
        
        /* Send each item of array individually */
        /*printf("io send(%d,,%d): %s",
            fd,
            strlen(SvPV(*av_fetch(Reply_data, i, FALSE), n_a)),
            SvPV(*av_fetch(Reply_data, i, FALSE), n_a));*/
        printf("io send(sock=%d,size=%d): %s",
                fd, strlen(Buf), Buf);

        send(fd, Buf, strlen(Buf), 0);
        memset(Buf, 0, strlen(Buf));
    }
    
    if (Buf != NULL)
        free(Buf);
    Buf = NULL;
    
    av_clear(Reply_data); /* Clear array */

    /* Check if the script is finished with the connection */    
    sv_data = get_sv("close_connection", TRUE);
    D_Notice = SvIV(sv_data);
    if (D_Notice == 1)
    {
        sv_setiv(sv_data, 0);
        return 11;
    }

    return 0;
}

/* EOF */
