/*
 * Impost(er) by ziplock (http://ziplock.b4b0.net)
 * File: shcdefs.c (Manages shellcode definitions and handling)
 *
 * Comments: sickbeatz@hotmail.com
 * IRC: #b4b0 on EFNet
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */

#include "impost.h"
#include "messages.h"
#include "shextra.h"

int show_matching_opcodes = 0;
extern struct config_options config_opt[];
static struct shellcodes Shellcode_Definitions[50];

/*
** Parse shellcode definition file and allocate defined data
*/
int SHC_Parse_Defs_File()
{
   char *Line, *p, *q, *Hex_data, *Desc, *Temp_str;
   int Line_No        = 0;
   int SHC_Defs_count = 0;

   FILE *SHC_Defs_File;

    /* Open shellcode definitions read only */
   SHC_Defs_File = fopen(config_opt[C_OPT_SHELLCODE_DEFS].option,
			 F_OPEN_READONLY);

    /* Nope */
   if (SHC_Defs_File == NULL)
     {
        fprintf(stderr, SHCDEFS_ERROR_OPEN);

        perror("fopen()");
        return -1;

     }

    /* Allocate Line, Hex_data, Desc and Temp_str */
   if (!(Line = (char *)malloc(SHC_LINE_SIZE)) ||
       !(Hex_data = (char *)malloc(SHC_LINE_SIZE)) ||
       !(Desc = (char *)malloc(SHC_LINE_SIZE)) ||
       !(Temp_str = (char *)malloc(SHC_LINE_SIZE)))
     {
        fprintf(stderr, ERROR_MALLOC);

        return -1;
     }

    /* Get next line unless zero is returned*/
   while (fgets(Line, SHC_LINE_SIZE, SHC_Defs_File) != 0)
     {
        Line_No++; /* Inc. line number */

        /* Reset variables */
        memset(Hex_data, 0, strlen(Hex_data));
        memset(Desc, 0, strlen(Desc));

        /* Ignore comments and NULL lines */
        if (*Line == '#' || !*Line)
	  continue;

        /* Terminate line from here on, it's a comment */
        if ((p = strchr(Line, '#')))
	  *p = 0;

        /* NULL out next line character */
        if ((p = strchr(Line, '\n')))
	  *p = 0;

        /* Ignore empty/NULL lines */
        q = Line;
        if (Line[0] == '\0' || *q == '\0')
	  continue;

        /* Syntax: (<hex data>,<desc>) -  this will get
           ,<desc>) so increase start by 1 to remove , */
        if ((p = strchr(Line, ',')))
	  {

	     *p++;

	     strcpy(Desc, p); /* Description */

	     while (Desc[0] == ' ')
	       Desc++;

            /* Remove )* so we got only <desc> which is
               what we want */
	     if (strtok(Desc, ")") == NULL)
	       {
		  fprintf(stderr, SHCDEFS_BADSYNTAX, Line_No);

		  continue;
	       }

	  }
	else
	  {
             fprintf(stderr, SHCDEFS_BADSYNTAX, Line_No);

             continue;

	 }

        /* From our syntax, get (<hex data> */
        strcpy(Temp_str, Line);
        strtok(Temp_str, ",");

        /* Remove initial ( so we get only <hex data>,
           Check and make sure that the first character is '(' to be safe */
        if (Temp_str[0] == '(')
	  Temp_str++;

        /* Since we're comparing Hexidecimals, make sure
           we're not getting other crap too */
        strcpy(Hex_data, Temp_str);

        if (Hex_data && Desc)
	  {
	     sprintf(Shellcode_Definitions[SHC_Defs_count].ascii_hex_pattern,
		     "%s", Hex_data);

	     sprintf(Shellcode_Definitions[SHC_Defs_count].Description,
		     "%s", Desc);

	     Shellcode_Definitions[SHC_Defs_count].Defined = 1;

	     SHC_Defs_count++;
	  }
     }

   fclose(SHC_Defs_File);

   return 0;
}
/*
** Check if the buffer actually does have
** opcode instructions, if it does return 1 if not return 0
*/
int SHC_Has_X86_Opcode(char *buffer)
{
   int i, c = 0, b = 0, x, t;
   char *pointer = (char *)malloc(52);

    /* Start scanning buffer */
   for (i = 0; i < strlen(buffer); i++)
     {
        for (x = 0; x < X86_OPCODES; x++)
	  {
            /* Clear pointer */
	     memset(pointer, 0, 52);

            /* Put together pointer to compare with the current
               opcode */
	     for (t = 0; t < x86_common[x].n_codes; t++)
	       {
		  pointer[t] = buffer[i+t];
	       }

            /* Compare pointer with current opcode */
	     if (!strncmp(pointer, x86_common[x].operation_code,
			  x86_common[x].n_codes))
	       {
		  b += x86_common[x].n_codes; /* Increase matching bytes
					       by size of the current opcode */

		  c++; /* Increase match count */
		  if (show_matching_opcodes)
                    printf(".d Found: (%s) %s -- start position: %d\n",
			   x86_common[x].shellcode,
			   x86_common[x].assembly_code, i);
	       }
	  }
     }

    /* If the bytes matched isn't greater than the opcodes
       matched, this buffer is not shellcode */
   return (b > c) ? 1 : 0;
}

/*
** Convert ASCII hex format to hex buffer
*/
char *SHC_PutTogetherBuffer(char *source)
{
   char *Buffer, *Tmp;
   int Buffer_size = (strlen(source) * 2);
   int C_hex, i;

   if (!(Buffer = (char *)malloc(Buffer_size)) ||
       !(Tmp = (char *)malloc(2)))
     {
        fprintf(stderr, ERROR_MALLOC);

        exit(1);
     }

   memset(Buffer, 0, Buffer_size);
   for (i = 0; i < strlen(source); i++)
     {
        /* Ignore spaces */
        if (source[i] == ' ')
	  continue;

        Tmp[0] = source[i];
        Tmp[1] = source[i+1];

        C_hex = (char)SHC_Str2H(Tmp) & 0xff;

        sprintf(Buffer, "%s%c", Buffer, C_hex);

        i++;
     }

   return Buffer;
}

/*
** Convert ASCII to Hex... I jacked this function from code of mine
** I made a few years ago called gH-cgi.c.. I found it on
** packetstorm a few months ago.
*/
char SHC_Str2H(char *Str)
{
   register char ret;

   if (Str[0] >= 'A')
     ret = ((Str[0] & 0xdf) - 'A') + 10;
   else
     ret = (Str[0] - '0');

   ret *= 16;

   if (Str[1] >= 'A')
     ret += ((Str[1] & 0xdf) - 'A') + 10;
   else
     ret += (Str[1] - '0');

   return ret;
}

/*
** Check if buffer has been mutated by ADM-Mutant
** Right now only Intel
*/
int SHC_CheckADMmutant(char *TestBuffer)
{
   char *Code;
   char *Buffer;
   int  Code_matches = 0;
   int  Junk_len;
   int  Junk_dyn;
   int  Junk_dynint;
   int  Current;
   int  Contains_Common[5];
   int  i;

   if (!(Code = (char *) malloc(strlen(TestBuffer))))
     {
        printf(ERROR_MALLOC);

        exit(1);
     }

   Buffer = TestBuffer;

#ifdef DEBUG_MUTATE
   printf(".d ADM Mutate test: checking for Mutate's Morph padding\n");
#endif
   for (i = 0; i < strlen(Buffer); i++)
     {
        for (Current = 0; Current < IA32_JUNKS; Current++)
	  {
	     Junk_len = intel_njunk[Current].len;
	     Junk_dyn = intel_njunk[Current].dyn;
	     Junk_dynint = intel_njunk[Current].dynint;

            /* Check for Mutate's Morph padding */
	     if (intel_njunk[Current].morph)
	       {
		  if ((!Junk_dyn && !strncmp(Buffer, intel_njunk[Current].code, Junk_len)) ||
		      (Junk_dyn && !strncmp(Buffer, intel_njunk[Current].code, Junk_dynint)))
		    {
		       Code_matches++;

		       if ((intel_njunk[Current].len-1) < strlen(Buffer))
			 Buffer += intel_njunk[Current].len - 1;
		    }
	       }
	  }
        Buffer++;
     }

    /* According to my research and statistics, if it is
       polymoprhed by ADM's Mutate, then the result will
       never be less than 4.5 or more than 7 */
#ifdef DEBUG_MUTATE
   printf(".d ADM Mutate test: %d (buffer length / code matches)\n",
	  Code_matches ? strlen(TestBuffer)/Code_matches : 0);
#endif

    /* We don't want a float point exception to occur */
   if (Code_matches == 0)
     return 0;

   if ((strlen(TestBuffer)/Code_matches) < 4 ||
       (strlen(TestBuffer)/Code_matches) > 7)
     return 0;

    /* ADM's Mutate also always has the following characters
       present in it's 'mutated' data: [, ], _, @, ? --- Intel Only */
   for (i = 0; i < strlen(TestBuffer); i++)
     {
        switch (TestBuffer[i])
	  {
	   case '[':
	     Contains_Common[0] = 1;
	     break;
	   case ']':
	     Contains_Common[1] = 1;
	     break;
	   case '/':
	     Contains_Common[2] = 1;
	     break;
	   case '?':
	     Contains_Common[3] = 1;
	     break;
	   case '_':
	     Contains_Common[4] = 1;
	     break;
	   default:
	     break;
	  }
     }

    /* Make sure all the common characters were found */
   for (i = 0; i < 4; i++)
     if (Contains_Common[i] != 1)
       return 0;

    /* It appears to be polymorphed by ADM's Mutate :) */
   return 1;
}

/*
** Compare the buffer with known shellcode patterns defined in
** impost.shellcodes
*/
int SHC_Check_If_Known(char *Buffer)
{
   char *Def_Buffer = (char *)malloc(256);
   int i = 0;
   int Found = 0;

    /* Check to see if the shellcode is polymorphic
       based on the ADM-Mutant engine */
   if ((i = SHC_CheckADMmutant(Buffer)))
     {
        if (i == 1)
	  printf(SP_ADM_MUTATED_INTEL);

        return 1;
     }

    /* Make sure we're not searching through empty strings, some
       systems are weird like that */
   while (Shellcode_Definitions[i].Defined == 1)
     {
        Def_Buffer = SHC_PutTogetherBuffer(Shellcode_Definitions[i].ascii_hex_pattern);
        if (strstr(Buffer, Def_Buffer))
	  {
            /* word */
	     printf(SP_SHELLCODE_FOUND,
                    Shellcode_Definitions[i].Description);

	     Found = 1;
	     //return 1; /* Return Identifier for which item in Shellcode_Definitions */
	  }

        i++; /* Try next */
     }

   return Found;
}

/* EOF */
